/*
   Filename: MealBrowser.cpp

   The MealBrowser class represents a browser like
   object to browse and manipulate Meals

   Last Updated: 04/21/04
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include "MealBrowser.h"

Fl_Window *recWindow;

//-------------------------------------------------------------------------------------
MealBrowser::MealBrowser(DataStore *ds)
{
    mealNameInput = new Fl_Input(90, 15, 150, 25, "Meal Name:");
    rlist = new Fl_Hold_Browser(260, 15, 265, 220, "Recipes");
    ralist = new Fl_Hold_Browser(15, 85, 230, 240, "Added Recipes");
    {
		Fl_Button *o = new Fl_Button(260, 300, 85, 25, "View Recipe");
		o->callback(cb_viewRecb, this);
	}
    addB = new Fl_Button(260, 255, 85, 25, "Add");
	addB->callback(cb_Addb, this);
	
	deleteB = new Fl_Button(445, 255, 85, 25, "Delete");
	deleteB->callback(cb_Delb, this);
	
    okB = new Fl_Button(355, 300, 80, 25, "Ok");
	okB->callback(cb_Validate, this);
	
    numOfPeopleInput = new Fl_Int_Input(90, 45, 150, 25, "# of People");
    cancelB = new Fl_Button(445, 300, 85, 25, "Cancel");
	cancelB->callback(cb_Cancelb, this);

    //new Fl_Button(355, 300, 80, 25, "Add Meal");

	theRecipes = ds->getRecipes();
	theMeals = ds->getMeals();

	db = ds;
	
	updateView();
	view = false;
}
//-------------------------------------------------------------------------------------
void destroy_rec_window( Fl_Widget *, void * )
{
	recWindow->~Fl_Window();
}
//-------------------------------------------------------------------------------------
MealBrowser::~MealBrowser() {
//  delete window;
}
//-------------------------------------------------------------------------------------
void MealBrowser::updateView()
{
	string temp;
	for(unsigned int i=0; i<theRecipes->size(); i++)
	{
		temp = (*theRecipes)[i].getTitle();
	    rlist->add(temp.c_str());
	}

	doscale = true;
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Addb(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_Addb_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Addb_i(Fl_Widget *w)
{
	int selection = rlist->value();
	
	if(selection == 0) {
		fl_alert("Please make a selection before trying to add a recipe");
		return;
	}	 

	ralist->add(rlist->text(selection));

	vr.push_back((*theRecipes)[selection-1]);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Delb(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_Delb_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Delb_i(Fl_Widget *w)
{
	int selection = ralist->value();
	
	if(selection == 0) {
		fl_alert("Please make a selection before trying to delete a recipe");
		return;
	}	 

	ralist->remove(selection);

	vector<Recipe>::iterator deleteIterator;

	deleteIterator = vr.begin() + (selection-1);

	// deleting in the middle of a vector isn't very good,
	// and is certiantly not very fast. We may want to
	// consider going to a linked list.

	vr.erase( deleteIterator );
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Validate(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_Validate_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Validate_i(Fl_Widget *w)
{
	const char *name = mealNameInput->value();
	const char *people = numOfPeopleInput->value();
	float thePeople;

	list<string> scaledIngred;
	list<string> ingredL;

	float mul;

	stringstream s;

	string theName;

	if(name == NULL || !strcmp(name,""))
	{
		fl_alert("Before you can add a meal, it must have a name.");
		return;
	}

	if(people == NULL || !strcmp(people,""))
	{
		fl_alert("Before you can add a meal, it must have a number of servings.");
		return;
	}

	theName = name;

	thePeople = atof(people);
	
	s.clear();

	for(unsigned int i = 0; i < vr.size(); i++)
	{
		ingredL = vr[i].getIngredients();
		mul = thePeople / vr[i].getServings(); 
		// scaling code.
		list<string>::iterator p = ingredL.begin();
		
		while(p != ingredL.end())
		{
			string atemp = *p;
			string newguy = "";
			int pos= atemp.find(" ");
			if(pos != -1){
				float theNum = atof(atemp.c_str());
				theNum *= mul;	
				s << theNum;
				s >> newguy;
				s.clear();
				newguy = newguy + " " + atemp.substr(pos,atemp.size());
				scaledIngred.push_back(newguy);			
			}
			p++;
		}		
		vr[i].setIngredients(scaledIngred);
		scaledIngred.clear();
	}

	
	Meal b(vr, "", theName, thePeople);
	//theMeals.push_back(b);
	(*theMeals).push_back(b);

	ralist->clear();

	mealNameInput->value("");
	numOfPeopleInput->value("");

	vr.clear();

	goBacktoPlan_cb(w,this);	
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_viewRecb(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_viewRecb_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_viewRecb_i(Fl_Widget *w)
{
	const char *people = numOfPeopleInput->value();
	int selection = ralist->value();

	float mul;

	float temp;

	stringstream s;

	string servingsNum, temp2;

	list<string> ingredL;

	if(people == NULL || !strcmp(people,""))
	{
		fl_alert("This recipe's servings can not be scaled\nwithout knowing the number of people.");
		return;
	}
		
	if(selection == 0) {
		fl_alert("Please make a selection before trying to view a recipe");
		return;
	}

	temp = atof(people);

	recWindow = new Fl_Window (546, 347, "Recipe Viewer...");

    titleDisp = new Fl_Input(45, 20, 165, 25, "Title:");
	titleDisp->readonly(1);
	titleDisp->textsize(titleDisp->textsize()-1);

	categoryDisp = new Fl_Input(95, 55, 115, 25, "Category:");
    categoryDisp->readonly(1);
	categoryDisp->textsize(categoryDisp->textsize()-1);

    servingDisp = new Fl_Int_Input(95, 90, 115, 25, "Serving Size:");
	servingDisp->readonly(1);

	ingredSB = new Fl_Scrollbar(510, 20, 20, 115);
	ingredSB->labelsize(12);
    ingredSB->maximum(45);
	ingredSB->slider_size(0.50);
    ingredSB->box(FL_DOWN_BOX);
	ingredSB->callback(cb_scrollingred, this);

	ingredDisp = new Fl_Multiline_Input(215, 20, 295, 115, "Ingredients:");
	ingredDisp->align(FL_ALIGN_TOP);
	ingredDisp->wrap(48);
	ingredDisp->readonly(1);
	ingredDisp->textsize(ingredDisp->textsize()-1);

	
	{
		Fl_Scrollbar* o = directSB = new Fl_Scrollbar(510, 155, 20, 120);
		o->labelsize(12);
	    o->maximum(45);
		o->slider_size(0.50);
        o->box(FL_DOWN_BOX);        
		o->callback(cb_scrolldirect, this);

		directDisp = new Fl_Multiline_Input(5, 155, 505, 120, "Directions");
		directDisp->align(FL_ALIGN_TOP);
		directDisp->wrap(80);
		directDisp->readonly(1);
		directDisp->textsize(directDisp->textsize()-1);

	}

	
	mul = temp / vr[selection-1].getServings(); 	
	s << temp;
	s >> servingsNum;
	s.clear();

	ingredL = vr[selection-1].getIngredients();		
	list<string>::iterator p = ingredL.begin();

	if(doscale) 
	{
		
		// scaling code.		
		while(p != ingredL.end())
		{
			string atemp = *p;
			string newguy = "";
			int pos= atemp.find(" ");
			if(pos != -1){
				float theNum = atof(atemp.c_str());
				theNum *= mul;			
				s << theNum;
				s >> newguy;
				s.clear();
				newguy = newguy + atemp.substr(pos,atemp.size());
				temp2 = temp2 + newguy + '\n';		
			}
			p++;
		}
	//end scale code.
	}else 
	{
		// we still need the servings and ingredients.			
			while(p != ingredL.end())
			{
				temp2 = temp2 + *p + '\n';
				p++;
			}	
	}

	titleDisp->value(vr[selection-1].getTitle().c_str());
	categoryDisp->value(vr[selection-1].getCategory().c_str());
	servingDisp->value(servingsNum.c_str());
	ingredDisp->value(temp2.c_str());
	directDisp->value(vr[selection-1].getDirections().c_str());

	Fl_Button *dlg_button_ok = new Fl_Button (200, 300, 60, 25, "Ok");
	dlg_button_ok->callback(destroy_rec_window);
	
	recWindow->end();
	recWindow->show();	
	recWindow->set_modal();

}
//-------------------------------------------------------------------------------------
void MealBrowser::updateViewWithMeal()
{
	string temp;
	unsigned int i;

	stringstream s;

	Meal m = db->getMeal();

	vector<Recipe> recTemp;
	
	for(i=0; i<theRecipes->size(); i++)
	{
		temp = (*theRecipes)[i].getTitle();
	    rlist->add(temp.c_str());
	}

	recTemp = m.getRecipes();

	for(i=0; i<recTemp.size(); i++)
	{
		temp = recTemp[i].getTitle();
	    ralist->add(temp.c_str());
		vr.push_back(recTemp[i]);  // vr can't be null
	}

	s << m.getMealPeople();
	s >> temp;

	mealNameInput->value(m.getMealName().c_str());
	mealNameInput->readonly(1);
	numOfPeopleInput->value(temp.c_str());
	numOfPeopleInput->readonly(1);

	view = true;

	cancelB->label("Close");
	addB->deactivate();
	deleteB->deactivate();
	okB->deactivate();

	doscale = false;



}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Cancelb(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_Cancelb_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_Cancelb_i(Fl_Widget *w)
{
	mealNameInput->value("");
	numOfPeopleInput->value("");
	ralist->clear();

	vr.clear();

	if(view)
	{
		cancelB->label("Cancel"); // just in case it got reset by the viewer.
		addB->activate();
		deleteB->activate();
		okB->activate();
		mealNameInput->readonly(0);
		numOfPeopleInput->readonly(0);
		view = false;
	}

	goBacktoPlan_cb(w, this);
}
//-------------------------------------------------------------------------------------
void MealBrowser::update()
{
	string temp;
	rlist->clear();
	unsigned int i;

	for(i=0; i < theRecipes->size(); i++)
	{
		temp = (*theRecipes)[i].getTitle();
		rlist->add(temp.c_str());
	}
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_scrolldirect(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_scrolldirect_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_scrolldirect_i(Fl_Widget *w)
{
    int text_len;
    int newpos;
    text_len = directDisp->size();
	directDisp->take_focus();
    newpos =
        int (((float) directSB->value()) * ((float) text_len) /
             45.0);
    directDisp->position(newpos);
	directDisp->redraw();
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_scrollingred(Fl_Widget *w, void* data)
{
	((MealBrowser*)data)->cb_scrollingred_i(w);
}
//-------------------------------------------------------------------------------------
void MealBrowser::cb_scrollingred_i(Fl_Widget *w)
{
    int text_len;
    int newpos;
    text_len = ingredDisp->size();
	ingredDisp->take_focus();
    newpos =
        int (((float) ingredSB->value()) * ((float) text_len) /
             45.0);
    ingredDisp->position(newpos);
	ingredDisp->redraw();
}
//-------------------------------------------------------------------------------------
