#ifndef MEALBROWSER_H
#define MEALBROWSER_H

/*
   Filename: MealBrowser.h

   The MealBrowser class represents a browser like
   object to browse and manipulate Meals

   Last Updated: 04/21/04
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include <FL/Fl.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Button.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Scroll.H>

#include "DataStore.h"

class MealBrowser
{
    public:
		MealBrowser(DataStore *ds);
		~MealBrowser();		
		void updateViewWithMeal();
		void update();

    private:
		vector<Recipe> *theRecipes; // ALL recipes.
		vector<Meal> *theMeals;

		DataStore *db;

		vector<Recipe> vr; // recipe list for meals

		Fl_Input *mealNameInput;
		Fl_Int_Input *numOfPeopleInput;
		Fl_Hold_Browser *rlist;
		Fl_Hold_Browser *ralist;
		Fl_Button *cancelB;
		Fl_Button *addB;
		Fl_Button *deleteB;
		Fl_Button *okB;



		// below is for Viewing Recipe Window:
		Fl_Input *titleDisp;
		Fl_Input *categoryDisp;
		Fl_Int_Input *servingDisp;
		Fl_Multiline_Input *ingredDisp;
		Fl_Scrollbar *ingredSB; // each input needs a scroll bar.
		Fl_Multiline_Input *directDisp;
		Fl_Scrollbar *directSB; // each input needs a scroll bar.
		// end Recipe Window code.

		bool view;
		bool doscale;


		void updateView();

		static void cb_Addb(Fl_Widget *w, void* data);
		void cb_Addb_i(Fl_Widget *w);

		static void cb_Delb(Fl_Widget *w, void* data);
		void cb_Delb_i(Fl_Widget *w);

		static void cb_Validate(Fl_Widget *w, void* data);
		void cb_Validate_i(Fl_Widget *w);

		static void cb_viewRecb(Fl_Widget *w, void* data);
		void cb_viewRecb_i(Fl_Widget *w);

		static void cb_Cancelb(Fl_Widget *w, void* data);
		void cb_Cancelb_i(Fl_Widget *w);

		// custom scrollbar code since FLTK doesn't do it for us.
		static void cb_scrolldirect(Fl_Widget *w, void* data);
		void cb_scrolldirect_i(Fl_Widget *w);
		static void cb_scrollingred(Fl_Widget *w, void* data);
		void cb_scrollingred_i(Fl_Widget *w);


};

/* All callbacks must be static void, and can not be
   members of a class */
//static void about_cb(Fl_Widget *w, void *userdata);
extern void goBacktoPlan_cb(Fl_Widget*,void*); // iikh.cxx

#endif	/* MEALBROWSER_H */
