/*
   Filename: PlanBrowser.cpp

   The PlanBrowser class represents the a browser like
   object to browse and manipulate Recipes

   Last Updated: 04/22/04
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include "PlanBrowser.h"

Fl_Window *GroceryWindow;
Fl_Window *printWindow;

//-------------------------------------------------------------------------------------
PlanBrowser::PlanBrowser(DataStore *ds)
{
    planNameInput = new Fl_Input(90, 15, 150, 25, "Plan Name:");
    plist = new Fl_Hold_Browser(260, 15, 265, 220, "Plans");
    mlist = new Fl_Hold_Browser(15, 85, 230, 150, "Meals");
    {
		Fl_Button *o = new Fl_Button(15, 300, 90, 25, "View Meal");
		o->callback(cb_Viewb, this);
	}
    {
		Fl_Button *o = new Fl_Button(235, 255, 90, 25, "Add Plan");
		o->callback(cb_Addb, this);
	}
    {
		Fl_Button *o = new Fl_Button(115, 255, 90, 25, "Delete Meal");
		o->callback(cb_Delb, this);
	}
    {
		Fl_Button *o = new Fl_Button(335, 255, 90, 25, "Select Plan");
		o->callback(cb_selpb, this);
	}

    dateInput = new Fl_Input(90, 45, 150, 25, "Date:");
    {
		Fl_Button* o = new Fl_Button(435, 300, 90, 25, "Close");
		o->callback(back2_cb);
	}
    {
		Fl_Button* o = new Fl_Button(15, 255, 90, 25, "Add Meal");
		o->callback(gonext_cb);
	}

	{
		Fl_Button *o = new Fl_Button(235, 300, 90, 25, "Grocery List");
		o->callback(cb_grocb, this);
	}

	{
		Fl_Button *o = new Fl_Button(335, 300, 90, 25, "Print Plans");
		o->callback(cb_print, this);
	}
	{
		Fl_Button *o = new Fl_Button(435, 255, 90, 25, "Delete Plan");
		o->callback(cb_Delpb, this);
	}


	planNameInput->value("New Plan");
	dateInput->value("04/27/2004");
	theDataStore = ds;


	theMeals = theDataStore->getMeals();
	thePlans = theDataStore->getPlans();
}
//-------------------------------------------------------------------------------------
void destroy_window( Fl_Widget *, void * )
{
	printWindow->~Fl_Window();
}
//-------------------------------------------------------------------------------------
PlanBrowser::~PlanBrowser() {
//  delete window;
}
//-------------------------------------------------------------------------------------
void PlanBrowser::update()
{
	string temp;
	mlist->clear();
	unsigned int i;

	for(i=0; i < theMeals->size(); i++)
	{
		vm.push_back((*theMeals)[i]);
	}

	theMeals->clear();

	for(i=0; i < vm.size(); i++)
	{
		temp = vm[i].getMealName();
		mlist->add(temp.c_str());
	}
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Viewb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_Viewb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Viewb_i(Fl_Widget *w)
{
	int selection = mlist->value();
	
	if(selection == 0) {
		fl_alert("Please make a selection before trying to view a meal.");
		return;
	}	 

	theDataStore->storeMeal(vm[selection-1]);

	goMealUpdate_cb(w, this);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Delb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_Delb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Delb_i(Fl_Widget *w)
{
	int selection = mlist->value();	
	
	if(selection == 0) {
		fl_alert("Please make a selection before trying to delete a meal.");
		return;
	}	 


	mlist->remove(selection);

	vector<Meal>::iterator deleteIterator;

	deleteIterator = vm.begin() + (selection-1);

	// deleting in the middle of a vector isn't very good,
	// and is certiantly not very fast. We may want to
	// consider going to a linked list.

	vm.erase( deleteIterator );
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Addb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_Addb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Addb_i(Fl_Widget *w)
{
	stringstream s; 

	vector<Meal> vmTemp;

	for(unsigned int i = 0; i < vm.size(); i++)
		vmTemp.push_back(vm[i]);  // copy meals.


	string planNameString = planNameInput->value();
	string dateString = dateInput->value();
	
	Plan c(planNameString.c_str(), dateString.c_str(), vmTemp);
	mlist->clear();

	(*thePlans).push_back(c);

	planNameString = planNameString + " - " + dateString;

	plist->add(planNameString.c_str());

	vm.clear();

}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_selpb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_selpb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_selpb_i(Fl_Widget *w)
{
	int selection = plist->value();	

	string PlanName;
	string PlanDate;

	if(selection == 0) {
		fl_alert("Please make a selection before trying to show a plan.");
		return;
	}	 

	vector<Meal> vmTemp;
	
	PlanName = (*thePlans)[selection-1].getName();
	PlanDate = (*thePlans)[selection-1].getDate();

	vmTemp = (*thePlans)[selection-1].getMealList();

	vm = vmTemp;

	mlist->clear();

	planNameInput->value(PlanName.c_str());
	dateInput->value(PlanDate.c_str());
	
	for(unsigned int i = 0; i < vm.size(); i++)
		mlist->add(vm[i].getMealName().c_str());

}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Delpb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_Delpb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_Delpb_i(Fl_Widget *w)
{
	int selection = plist->value();	

	string PlanName;
	string PlanDate;

	if(selection == 0) {
		fl_alert("Please make a selection before trying to delete a plan.");
		return;
	}	 

	plist->remove(selection);

	vector<Plan>::iterator deleteIterator;

	deleteIterator = (*thePlans).begin() + (selection-1);

	// deleting in the middle of a vector isn't very good,
	// and is certiantly not very fast. We may want to
	// consider going to a linked list.

	(*thePlans).erase( deleteIterator );
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_grocb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_grocb_i(w);
}
//-------------------------------------------------------------------------------------
// Grocery List stuff
// Our goal is to go through each Plan, and each meal, and each recipe
// and add it to the list, if there are duplicates then update the 
// quantity.
//
// we have to do this in Plan because otherwise users can add and change
// the recipes and meals everywhere else and the program doesn't work
// with the grocery list there. 
//
void PlanBrowser::cb_grocb_i(Fl_Widget *w)
{
	vector<Meal> vmTemp;  // a vector to hold Meals	
	vector<Recipe> rTemp; // a vector to hold all recipes.
	list<string> intemp3; // to store recipe's ingredients
	float num = 0; // store the quantity of each ingredient

	list<string>::iterator p;

	stringstream s; // our C++ converter.

	unsigned int i, j, k, l;   //loop counters  it will be O(n^5)  :(

	for(i = 0; i < (*thePlans).size(); i++) // go through each plan
	{
		//get all the meals.
		vmTemp = (*thePlans)[i].getMealList(); // really it's getMealVector
		for(j = 0; j < vmTemp.size(); j++)
		{
			rTemp = vmTemp[j].getRecipes();  // ok we have our vector of recipes.
			for(k = 0; k < rTemp.size(); k++)
			{
				intemp3 = rTemp[k].getIngredients();
				p = intemp3.begin();
				while(p != intemp3.end())
				{					
					string atemp = *p;
					int pos= atemp.find(" ");

					if(pos != -1){
					
						num = atof(atemp.c_str());				
						string end = atemp.substr(pos,atemp.size());

						for(l = 0; l < ingredlist.size(); l++)
						{
							if(end == ingredlist[l])
							{
								// they match							 
								amount[l] += num;
								break;
							}
						}

						if(l == ingredlist.size()) // there was no match
						{
							ingredlist.push_back(end);
							amount.push_back(num);
						}							

					}
					num = 0;
					p++;
				}
			}
		}
	}	

	GroceryWindow = new Fl_Window(400,300,"Grocery List");

	Fl_Hold_Browser *gList = new Fl_Hold_Browser(15,15,265,250);

	Fl_Button *dlg_button_ok = new Fl_Button(300, 25, 80, 25, "Ok");
	dlg_button_ok->callback(cb_destroygwin, this);

	{
		Fl_Button *o = new Fl_Button(300, 60, 80, 25, "Save");
		o->callback(cb_savegrocb, this);
	}

	GroceryWindow->end();
	GroceryWindow->show();
	GroceryWindow->set_modal();

	string temp;
	string temp2;
	
	for(unsigned int q = 0; q < ingredlist.size(); q++)
	{
		s << amount[q];
		s >> temp2;
		s.clear();

		temp2 += " " + ingredlist[q];
		gList->add(temp2.c_str());

		temp2 = "";

	}


}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_destroygwin( Fl_Widget *w, void * data )
{
	((PlanBrowser*)data)->destroy_groc_window(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::destroy_groc_window( Fl_Widget *w)
{
	GroceryWindow->~Fl_Window();
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_print(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_print_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_print_i(Fl_Widget *w)
{
	printWindow = new Fl_Window(296, 178);
    
    { Fl_Box* o = new Fl_Box(10, 20, 275, 95, "Options");
      o->box(FL_EMBOSSED_FRAME);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    { Fl_Group* o = new Fl_Group(10, 20, 275, 95);
      { Fl_Round_Button* o = selected = new Fl_Round_Button(20, 30, 100, 25, "Selected");
		o->type(FL_RADIO_BUTTON);
		o->value(1);
      }
	  { Fl_Round_Button* o = all = new Fl_Round_Button(20, 80, 50, 25, "All");
		o->type(FL_RADIO_BUTTON);        
      }
      { Fl_Round_Button* o = range = new Fl_Round_Button(20, 55, 75, 25, "Range");
		o->type(FL_RADIO_BUTTON);
      }
      from = new Fl_Int_Input(155, 55, 35, 25, "From:");
      to = new Fl_Int_Input(230, 55, 35, 25, "To:");
      o->end();
    }
	{    
		Fl_Button *o = new Fl_Button(135, 135, 70, 25, "Print");
		o->callback(cb_doPrinting, this);    
	}
	{
		Fl_Button *o = new Fl_Button(215, 135, 70, 25, "Close");
		o->callback(destroy_window);    
	}
	
	printWindow->set_modal();
	printWindow->end();
	printWindow->show();
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_doPrinting(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_doPrinting_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_doPrinting_i(Fl_Widget *w)
{
	int selection = plist->value();	

	ofstream out("Plans.txt");

	vector<Meal> vmTemp;  // a vector to hold Meals	
	vector<Recipe> rTemp; // a vector to hold all recipes.
	list<string> intemp3; // to store recipe's ingredients

	unsigned int i, j, k;

	stringstream s;
	string temp;

	list<string>::iterator p;
	
	if(selected->value())
	{
		if(selection==0)
		{
			fl_alert("Please make a selection first!");
			return;
		}

		out << (*thePlans)[selection-1].getName() + " ";
		out << (*thePlans)[selection-1].getDate() + '\n';		

		vmTemp = (*thePlans)[selection-1].getMealList(); // really it's getMealVector
		for(j = 0; j < vmTemp.size(); j++)
		{
			out << "Meal: " + vmTemp[j].getMealName() + '\n';
			out << "====================================================================\n";
			rTemp = vmTemp[j].getRecipes();  // ok we have our vector of recipes.
			for(k = 0; k < rTemp.size(); k++)
			{
				out << rTemp[k].getTitle() + '\n';
				out << rTemp[k].getCategory() + '\n';

				s << vmTemp[j].getMealPeople();
				s >> temp;
				s.clear();

				out << "Serves: " + temp + '\n';
				out << "Ingredients\n---------------\n";
				intemp3 = rTemp[k].getIngredients();
				p = intemp3.begin();
				while(p != intemp3.end())
				{	
					out << *p + '\n';					
					p++;
				}
				out << "---------------\nDirections\n";
				out << rTemp[k].getDirections() + '\n';
			}
		}
		//selected
	}
	else if(range->value())
	{
		// do some range checking.

		int a = atoi(from->value());
		unsigned int b = atoi(to->value());

		if(a < 0) a = 0;
		if(b > thePlans->size()) b = thePlans->size();

		for(i = a; i < b; i++) // go through each plan
		{
		//get all the meals.
			out << (*thePlans)[i].getName() + " ";
			out << (*thePlans)[i].getDate() + '\n';

			vmTemp = (*thePlans)[i].getMealList(); // really it's getMealVector
			for(j = 0; j < vmTemp.size(); j++)
			{
				out << "Meal: " + vmTemp[j].getMealName() + '\n';
				out << "====================================================================\n";
				rTemp = vmTemp[j].getRecipes();  // ok we have our vector of recipes.
				for(k = 0; k < rTemp.size(); k++)
				{
					out << rTemp[k].getTitle() + '\n';
					out << rTemp[k].getCategory() + '\n';

					s << vmTemp[j].getMealPeople();
					s >> temp;
					s.clear();

					out << "Serves: " + temp + '\n';
					out << "Ingredients\n---------------\n";
					intemp3 = rTemp[k].getIngredients();
					p = intemp3.begin();
					while(p != intemp3.end())
					{	
						out << *p + '\n';					
						p++;
					}
					out << "---------------\nDirections\n";
					out << rTemp[k].getDirections() + '\n';
				}
			}
		}
	}
	else 
	{ // all
		for(i = 0; i < (*thePlans).size(); i++) // go through each plan
		{
		//get all the meals.
			out << (*thePlans)[i].getName() + " ";
			out << (*thePlans)[i].getDate() + '\n';

			vmTemp = (*thePlans)[i].getMealList(); // really it's getMealVector
			for(j = 0; j < vmTemp.size(); j++)
			{
				out << "Meal: " + vmTemp[j].getMealName() + '\n';
				out << "====================================================================\n";
				rTemp = vmTemp[j].getRecipes();  // ok we have our vector of recipes.
				for(k = 0; k < rTemp.size(); k++)
				{
					out << rTemp[k].getTitle() + '\n';
					out << rTemp[k].getCategory() + '\n';
			
					s << vmTemp[j].getMealPeople();
					s >> temp;
					s.clear();

					out << "Serves: " + temp + '\n';
					out << "Ingredients\n---------------\n";
					intemp3 = rTemp[k].getIngredients();
					p = intemp3.begin();
					while(p != intemp3.end())
					{	
						out << *p + '\n';					
						p++;
					}
					out << "---------------\nDirections\n";
					out << rTemp[k].getDirections() + '\n';
				}
			}
		}
	}
	
	out.close();

	fl_alert("The plans have been printed.");

	destroy_window(w, this);


}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_savegrocb(Fl_Widget *w, void* data)
{
	((PlanBrowser*)data)->cb_savegrocb_i(w);
}
//-------------------------------------------------------------------------------------
void PlanBrowser::cb_savegrocb_i(Fl_Widget *w)
{
	ofstream out("Grocery.txt");

	out << "Grocery List\n";

	for(unsigned int i = 0; i < ingredlist.size(); i++)
	{
		out << amount[i] << " " << ingredlist[i] << '\n';
	}


	
	out.close();

	fl_alert("The grocery list have been saved to \"Grocery.txt\".");

	cb_destroygwin(w, this);
}






