/*
   Filename: RecipeBrowser.cpp

   The RecipeBrowser class represents the a browser like
   object to browse and manipulate Recipes

   Last Updated: 04/24/04
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include "RecipeBrowser.h"

Fl_Window *searchWindow;

//-------------------------------------------------------------------------------------
RecipeBrowser::RecipeBrowser(DataStore *db)
{
    title = new Fl_Input(45, 20, 165, 25, "Title:");
	title->readonly(1);
	title->textsize(title->textsize()-1);

	category = new Fl_Input(95, 55, 115, 25, "Category:");
    category->readonly(1);
	category->textsize(category->textsize()-1);

    serving = new Fl_Int_Input(95, 90, 115, 25, "Serving Size:");
	serving->readonly(1);

	ingredSB = new Fl_Scrollbar(510, 20, 20, 115);
	ingredSB->labelsize(12);
    ingredSB->maximum(45);
	ingredSB->slider_size(0.50);
    ingredSB->box(FL_DOWN_BOX);
	ingredSB->callback(cb_scrollingred, this);

	ingredI = new Fl_Multiline_Input(215, 20, 295, 115, "Ingredients:");
	ingredI->align(FL_ALIGN_TOP);
	ingredI->textsize(ingredI->textsize()-1);
	ingredI->type(FL_MULTILINE_INPUT_WRAP);
	ingredI->wrap(48);
	ingredI->readonly(1);

	{
		Fl_Scrollbar* o = directSB = new Fl_Scrollbar(510, 155, 20, 120);
		o->labelsize(12);
	    o->maximum(45);
		o->slider_size(0.50);
        o->box(FL_DOWN_BOX);        
		o->callback(cb_scrolldirect, this);

		directI = new Fl_Multiline_Input(5, 155, 505, 120, "Directions");
		directI->align(FL_ALIGN_TOP);	
		directI->textsize(directI->textsize()-1);
		directI->type(FL_MULTILINE_INPUT_WRAP);
		directI->wrap(80);
		directI->readonly(1);

		
	}

    addbut = new Fl_Button(105, 315, 80, 25, "Add");
	addbut->callback(cb_Add, this);
    modbut = new Fl_Button(195, 315, 80, 25, "Modify");
	modbut->callback(cb_Mod, this);
    delbut = new Fl_Button(285, 315, 80, 25, "Delete");
	delbut->callback(cb_Del, this);
	closebut = new Fl_Button(445, 315, 75, 25, "Close");
	closebut->callback(cb_Close,this);
	nextbut = new Fl_Button(195, 280, 80, 25, "@>");
	nextbut->callback(cb_Next, this);
	prevbut = new Fl_Button(105, 280, 80, 25, "@<");
	prevbut->callback(cb_Prev, this);
	
    searchbut = new Fl_Button(285, 280, 80, 25, "Search...");
	searchbut->callback(cb_search,this);

	viewingRecipeNum = 0;

	theRecipes = db->getRecipes();
	maxRec = theRecipes->size();
	updateView();

	theDataStore = db;

	dFlag = false;

}
//-------------------------------------------------------------------------------------
RecipeBrowser::~RecipeBrowser() 
{
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::updateView()
{
	string temp, temp2;
	string directionsS;
	string cat;

	stringstream s;

	list<string> ingredL;

	temp = "";

	string titleS = (*theRecipes)[viewingRecipeNum].getTitle();
	float servingN = (*theRecipes)[viewingRecipeNum].getServings();
	directionsS = (*theRecipes)[viewingRecipeNum].getDirections();
	cat = (*theRecipes)[viewingRecipeNum].getCategory();

	ingredL = (*theRecipes)[viewingRecipeNum].getIngredients();

	list<string>::iterator p = ingredL.begin();
	while(p != ingredL.end())
	{
		temp2 = temp2 + *p + '\n';
		p++;
	}



	s << servingN;
	s >> temp;

	title->value(titleS.c_str());
	category->value(cat.c_str());
	serving->value(temp.c_str());
	ingredI->value(temp2.c_str());

	directI->value(directionsS.c_str());

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Next(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Next_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Prev(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Prev_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Next_i(Fl_Widget *w)
{
	viewingRecipeNum++;
	if(viewingRecipeNum == maxRec) viewingRecipeNum = 0;
	updateView();

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Prev_i(Fl_Widget *w)
{
	viewingRecipeNum--;
	if(viewingRecipeNum < 0) viewingRecipeNum = maxRec-1;
	updateView();

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Del(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Del_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Del_i(Fl_Widget *w)
{
	int choice = fl_ask("Are you sure you want to delete this recipe?");

	if(choice == 0) return;

	// handle yes choice.

	vector<Recipe>::iterator deleteIterator;

	deleteIterator = theRecipes->begin() + viewingRecipeNum;

	// deleting in the middle of a vector isn't very good,
	// and is certiantly not very fast. We may want to
	// consider going to a linked list.

	theRecipes->erase( deleteIterator );

	dFlag = true;  // modification.

	// go to the next recipe.

	maxRec--;

	viewingRecipeNum++;
	if(viewingRecipeNum >= maxRec) viewingRecipeNum = 0;
	updateView();


	return;

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Add(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Add_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Add_i(Fl_Widget *w)
{
	title->value("New Recipe");
	serving->value("0");
	category->value("");
	ingredI->value("Type ingredients here.");
	directI->value("Type directions here. When finished, click Save.");

	addbut->label("Cancel");
	addbut->callback(cb_Cancel, this);

	modbut->label("Save");
	modbut->callback(cb_Save, this);

	setActive(false);

	setReadWriteAll();

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::setReadWriteAll()
{
	title->readonly(0);
	serving->readonly(0);
	category->readonly(0);
	ingredI->readonly(0);
	directI->readonly(0);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Save(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Save_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Save_i(Fl_Widget *w)
{
	stringstream s;

	const char *pcsTitle;  // pcs = pointer to C string
	const char *pcsServing;
	const char *pcsCategory;
	const char *pcsIngred;
	const char *pcsDirect;

	int pos;

	string theTitle;
	float theServing;
	string theCategory;
	string temp;
	string theDirect;

	list<string> theIngred;

	title->readonly(1);
	serving->readonly(1);
	category->readonly(1);
	ingredI->readonly(1);
	directI->readonly(1);

	dFlag = true;


	pcsTitle = 	title->value();
	pcsServing = serving->value();
	pcsCategory = category->value();
	pcsIngred = ingredI->value();
	pcsDirect = directI->value();

	s << pcsServing;
	s >> theServing;

	theTitle = pcsTitle;
	theCategory = pcsCategory;

	temp = pcsIngred;


	pos = temp.find("\n");

	while(pos != -1)
	{		
		theIngred.push_back(temp.substr(0,pos));
		temp = temp.erase(0,pos+1);
		pos = temp.find("\n");
	}

	theDirect = pcsDirect;

	Recipe a(theTitle, theCategory, theServing, theIngred, theDirect);
	theRecipes->push_back(a);

	maxRec++;

	setActive(true);

	modbut->label("Modify");
	modbut->callback(cb_Mod,this);
	addbut->label("Add");
	addbut->callback(cb_Add, this);

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Mod(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Mod_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Mod_i(Fl_Widget *w)
{

	setActive(false);
	// go to the next recipe.

	setReadWriteAll();

	modbut->label("Save");
	modbut->callback(cb_Save2, this);
	addbut->label("Cancel");
	addbut->callback(cb_Cancel, this);


}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Close(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Close_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Close_i(Fl_Widget *w)
{
	if(dFlag)
	{
		int choice = fl_ask("You have modified a recipe, would you like\nto save the changes to Recipes.txt?");

		if(choice == 1)
			theDataStore->writeRecipeFile();
	}

	dFlag = false;
	back_cb(w, this);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Cancel(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Cancel_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Cancel_i(Fl_Widget *w)
{
	// reactivate the stuff.

	modbut->label("Modify");
	modbut->callback(cb_Mod,this);

	setActive(true);

	addbut->label("Add");
	addbut->callback(cb_Add, this);

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_search(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_search_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_search_i(Fl_Widget *w)
{
	searchWindow = new Fl_Window(415, 400, "Search...");  // y was 435
	{ 
		Fl_Box* o = new Fl_Box(15, 20, 385, 105, " Query:");  // change to 140 for sbutton
		o->box(FL_EMBOSSED_BOX);
		o->align(FL_ALIGN_TOP_LEFT);
    
	}	
    
	skeyword = new Fl_Input(25, 45, 365, 25, " Search for:");      
    	skeyword->callback(cb_doSearch, this);
	skeyword->align(FL_ALIGN_TOP_LEFT);
	skeyword->when(FL_WHEN_CHANGED);
	{ 
		Fl_Button* o = new Fl_Button(295, 85, 95, 25, "Cancel");      
		o->callback(destroy_search);
	}
	
	resultsbox = new Fl_Box(15, 145, 385, 240, " Results");
	resultsbox->box(FL_EMBOSSED_FRAME);    
	resultsbox->align(FL_ALIGN_TOP_LEFT);
	resultsbox->deactivate();    

	slist = new Fl_Hold_Browser(25, 155, 365, 185);
	slist->deactivate();

	selbut = new Fl_Button(295, 350, 85, 25, "Select");
	selbut->callback(cb_selbut, this);
	selbut->deactivate();  

	searchWindow->set_modal();
	searchWindow->end();
	searchWindow->show();
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Save2(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_Save2_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_Save2_i(Fl_Widget *w)
{
	stringstream s;

	const char *pcsTitle;  // pcs = pointer to C string
	const char *pcsServing;
	const char *pcsCategory;
	const char *pcsIngred;
	const char *pcsDirect;

	int pos;

	string theTitle;
	float theServing;
	string theCategory;
	string temp;
	string theDirect;

	list<string> theIngred;

	title->readonly(1);
	serving->readonly(1);
	category->readonly(1);
	ingredI->readonly(1);
	directI->readonly(1);

	dFlag = true;


	pcsTitle = 	title->value();
	pcsServing = serving->value();
	pcsCategory = category->value();
	pcsIngred = ingredI->value();
	pcsDirect = directI->value();

	s << pcsServing;
	s >> theServing;

	theTitle = pcsTitle;
	theCategory = pcsCategory;

	temp = pcsIngred;


	pos = temp.find("\n");

	while(pos != -1)
	{
		pos = temp.find("\n");
		theIngred.push_back(temp.substr(0,pos));
		temp = temp.erase(0,pos+1);
	}




	theDirect = pcsDirect;

	(*theRecipes)[viewingRecipeNum].setTitle(theTitle);
	(*theRecipes)[viewingRecipeNum].setCategory(theCategory);
	(*theRecipes)[viewingRecipeNum].setServings(theServing);
	(*theRecipes)[viewingRecipeNum].setIngredients(theIngred);
	(*theRecipes)[viewingRecipeNum].setDirections(theDirect);

	setActive(true);

	modbut->label("Modify");
	modbut->callback(cb_Mod,this);
	addbut->label("Add");
	addbut->callback(cb_Add, this);

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::setActive(bool flag)
{
	if(flag)
	{
		closebut->activate();
		nextbut->activate();
		prevbut->activate();
		searchbut->activate();
		delbut->activate();
	}
	else 
	{
		closebut->deactivate();
		nextbut->deactivate();
		prevbut->deactivate();
		searchbut->deactivate();
		delbut->deactivate();
	}

}
//-------------------------------------------------------------------------------------
void destroy_search(Fl_Widget *, void *userdata)
{
	searchWindow->~Fl_Window();

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_doSearch(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_doSearch_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_doSearch_i(Fl_Widget *w)
{
	resultsbox->activate();
	slist->activate();
	selbut->activate();

	slist->clear();

	string s = skeyword->value();

	for(unsigned int i = 0; i < (*theRecipes).size(); i++)
	{
		if((*theRecipes)[i].containsKeyword(s))
			slist->add((*theRecipes)[i].getTitle().c_str(), (void*)i);
	}
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_selbut(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_selbut_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_selbut_i(Fl_Widget *w)
{
	int selection = slist->value();

	if(selection == 0) { 
		fl_alert("Please make a selection");
		return;
	}

	int recnum = (int) slist->data(selection);

	viewingRecipeNum = recnum;

	destroy_search(w, this);
	updateView();

}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_scrolldirect(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_scrolldirect_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_scrolldirect_i(Fl_Widget *w)
{
    int text_len;
    int newpos;
    text_len = directI->size();
	directI->take_focus();
    newpos =
        int (((float) directSB->value()) * ((float) text_len) /
             45.0);
    directI->position(newpos);
	directI->redraw();
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_scrollingred(Fl_Widget *w, void* data)
{
	((RecipeBrowser*)data)->cb_scrollingred_i(w);
}
//-------------------------------------------------------------------------------------
void RecipeBrowser::cb_scrollingred_i(Fl_Widget *w)
{
    int text_len;
    int newpos;
    text_len = ingredI->size();
	ingredI->take_focus();
    newpos =
        int (((float) ingredSB->value()) * ((float) text_len) /
             45.0);
    ingredI->position(newpos);
	ingredI->redraw();
}
//-------------------------------------------------------------------------------------
