#ifndef RECIPEBROWSER_H
#define RECIPEBROWSER_H

/*
   Filename: recipebrowser.h

   The RecipeBrowser class represents the a browser like
   object to browse and manipulate Recipes

   Last Updated: 04/13/04
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Button.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Scroll.H>

#include <sstream>



#include "DataStore.h"


using namespace std;


class RecipeBrowser
{
    public:
		RecipeBrowser(DataStore *db);
		~RecipeBrowser();		


    private:
		int viewingRecipeNum;
		int maxRec;
		vector<Recipe> *theRecipes;
		bool dFlag;

		DataStore *theDataStore;

		Fl_Input *title;
		Fl_Input *category;
		Fl_Input *serving;
		Fl_Multiline_Input *ingredI;
		Fl_Scrollbar *ingredSB; // each input needs a scroll bar.
		Fl_Multiline_Input *directI;
		Fl_Scrollbar *directSB; // each input needs a scroll bar.

		// The Recipe Browser search window variables
		// are below

		Fl_Hold_Browser* slist; // search list results
		Fl_Button *selbut;   // select button
		Fl_Box *resultsbox;  // results box
		Fl_Input *skeyword;

		// end of search window variables.
		
		Fl_Button *addbut;
		Fl_Button *delbut;
		Fl_Button *modbut;

		Fl_Button *closebut;
		Fl_Button *nextbut;
		Fl_Button *prevbut;
		Fl_Button *searchbut;
		
		void updateView();

		void setReadWriteAll(); // set widgets to read/write mode.
		void setActive(bool flag);

		// all of our callbacks.

		static void cb_Next(Fl_Widget *w, void* data);
		void cb_Next_i(Fl_Widget *w);
		static void cb_Prev(Fl_Widget *w, void* data);
		void cb_Prev_i(Fl_Widget *w);
		static void cb_Del(Fl_Widget *w, void* data);
		void cb_Del_i(Fl_Widget *w);
		static void cb_Add(Fl_Widget *w, void* data);
		void cb_Add_i(Fl_Widget *w);
		static void cb_Save(Fl_Widget *w, void* data);
		void cb_Save_i(Fl_Widget *w);
		static void cb_Save2(Fl_Widget *w, void* data);
		void cb_Save2_i(Fl_Widget *w);
		static void cb_Mod(Fl_Widget *w, void* data);
		void cb_Mod_i(Fl_Widget *w);
		static void cb_Close(Fl_Widget *w, void* data);
		void cb_Close_i(Fl_Widget *w);
		static void cb_Cancel(Fl_Widget *w, void* data);
		void cb_Cancel_i(Fl_Widget *w);
		static void cb_search(Fl_Widget *w, void* data);
		void cb_search_i(Fl_Widget *w);
		static void cb_doSearch(Fl_Widget *w, void* data);
		void cb_doSearch_i(Fl_Widget *w);
		static void cb_selbut(Fl_Widget *w, void* data);
		void cb_selbut_i(Fl_Widget *w);

		// custom scrollbar code since FLTK doesn't do it for us.
		static void cb_scrolldirect(Fl_Widget *w, void* data);
		void cb_scrolldirect_i(Fl_Widget *w);
		static void cb_scrollingred(Fl_Widget *w, void* data);
		void cb_scrollingred_i(Fl_Widget *w);
		
};

void destroy_search(Fl_Widget *, void *userdata);

/* static void next_cb(Fl_Widget *, void *userdata); */

extern void back_cb(Fl_Widget*,void*); // iikh.cxx

#endif	/* RECIPEBROWSER_H */
