/*
   Filename: iikh.cpp

   The IIKH class represents the code for the GUI and and also
   the main object.

   Last Updated: 04/22/2004
   Written by: Erik Waibel, Devin Smith, James Wong, Phan Su.

*/

#include "iikh.h"

Fl_Window *G_win = 0;

#define W 536
#define H 347

//extern int FL_NORMAL_SIZE;

void back_cb(Fl_Widget*,void*) { G_wiz->prev(); }
void back2_cb(Fl_Widget*,void*) { G_wiz->prev(); G_wiz->prev(); }
void next_cb(Fl_Widget*,void*) { G_wiz->next(); }
void done_cb(Fl_Widget*,void*) { exit(0); }


//-------------------------------------------------------------------------------------
IIKH::IIKH()
{
	Fl_Box *box = new Fl_Box(FL_DOWN_BOX,180,35,335,255,NULL);
	Fl_Box *label = new Fl_Box(FL_PLASTIC_UP_BOX,15,15,150,60,"Interactive Intelligent\nKitchen Helper");
	label->labeltype(FL_EMBOSSED_LABEL);
	Fl_Button *br = new Fl_Button(15, 100, 150, 25, "Browse Recipes...");
	br->callback(next_cb);	
	Fl_Button *bp = new Fl_Button(15, 130, 150, 25, "Browse Plans...");
	bp->callback(gotoPlans_cb);
	Fl_Button *help = new Fl_Button(15, 220, 150, 25, "Help...");
	help->callback(help_cb);
	Fl_Button *about = new Fl_Button(15, 260, 150, 25, "About IIKH...");
	about->callback(about_cb);
	Fl_Button *quit = new Fl_Button(15, 300, 150, 25, "Quit IIKH");
	quit->callback(done_cb);
	box->image(new Fl_JPEG_Image("kitchen.jpg"));	
    
}
//-------------------------------------------------------------------------------------
void help_cb(Fl_Widget *w, void *userdata)
{
	Fl_Help_Dialog *Help_Window = new Fl_Help_Dialog;
	Help_Window->load("Doc/index.html");
	Help_Window->show();
}
//-------------------------------------------------------------------------------------
void about_cb(Fl_Widget *w, void *userdata)
{
	const char *credits = "IIKH v 1.0 (04/23/2004) Build 12\n\n" 
		"IIKH was written by Erik Waibel,\n" 
		"Phan Su, James Wong and \n"
		"Devin Smith.";
	
	dlg_window = new Fl_Window (270, 240, "About IIKH...");

	Fl_Box *mlo = new Fl_Box(FL_BORDER_BOX,10,10,250,150,NULL);
	mlo->label(credits);	
	
	Fl_Button *dlg_button_ok = new Fl_Button (100, 200, 60, 25, "Ok");
	dlg_button_ok->callback(destroy_dlg_window);

	dlg_window->set_modal();
	
	dlg_window->end();
	dlg_window->show();	
}
//-------------------------------------------------------------------------------------
void destroy_dlg_window( Fl_Widget *, void * )
{	
	dlg_window->~Fl_Window();
}
//-------------------------------------------------------------------------------------
int main(int argc, char **argv) {
	
	G_win = new Fl_Double_Window(W,H,"IIKH");
    G_wiz = new Fl_Wizard(0,0,W,H);

	DataStore *db = new DataStore();   // db = database
	db->parseRecipeFile();

	//FL_NORMAL_SIZE = 13; // change if font is too small or big

    // Wizard: page 1 Main Screen
    {
        Fl_Group *g = new Fl_Group(0,0,W,H);
		new IIKH();		
        g->end();
    }
    // Wizard: page 2 RecipeBrowser
    {
        Fl_Group *g = new Fl_Group(0,0,W,H);
		new RecipeBrowser(db);		
        g->end();
    }
    // Wizard: page 3 PlanBrowser
    {
        Fl_Group *g = new Fl_Group(0,0,W,H);
        pb = new PlanBrowser(db);
        g->end();
    }
    // Wizard: page 4 MealBrowser
    {
        Fl_Group *g = new Fl_Group(0,0,W,H);
		mb = new MealBrowser(db);		
        g->end();
    }

    G_wiz->end();
    G_win->end();
    G_win->show(argc, argv);
	Fl::scheme("plastic");  // or use NULL
	return Fl::run();
}
//-------------------------------------------------------------------------------------
void gonext_cb(Fl_Widget*,void*) 
{ 
	mb->update();
	G_wiz->next(); 
}
//-------------------------------------------------------------------------------------
void gotoPlans_cb(Fl_Widget*,void*) 
{ 
	pb->update();
	G_wiz->next(); G_wiz->next(); 
}
//-------------------------------------------------------------------------------------
void goBacktoPlan_cb(Fl_Widget*,void*) 
{ 
	pb->update();
	G_wiz->prev(); 
}
//-------------------------------------------------------------------------------------
void goMealUpdate_cb(Fl_Widget*,void* data) 
{ 
	mb->updateViewWithMeal();
	G_wiz->next(); 
}
//-------------------------------------------------------------------------------------
